@@ -1,20 +1,23 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2010 Oracle. All rights reserved.
+/*
+ * Copyright (c) 1998, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
 package org.eclipse.persistence.internal.xr;
 
 //javase imports
-import java.lang.reflect.Field;
+import java.util.HashSet;
 import java.util.Iterator;
+import java.util.Set;
 
 //java eXtension imports
 
@@ -22,7 +25,6 @@ import java.util.Iterator;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.internal.databaseaccess.Platform;
 import org.eclipse.persistence.internal.helper.ConversionManager;
-import org.eclipse.persistence.internal.helper.Helper;
 import org.eclipse.persistence.internal.indirection.BasicIndirectionPolicy;
 import org.eclipse.persistence.internal.xr.XRDynamicEntity;
 import org.eclipse.persistence.mappings.DatabaseMapping;
@@ -30,7 +32,6 @@ import org.eclipse.persistence.mappings.ForeignReferenceMapping;
 import org.eclipse.persistence.sessions.Login;
 import org.eclipse.persistence.sessions.Project;
 import static org.eclipse.persistence.internal.xr.XRDynamicClassLoader.COLLECTION_WRAPPER_SUFFIX;
-import static org.eclipse.persistence.internal.xr.XRDynamicEntity.XR_FIELD_INFO_STATIC;
 
 /**
  * <p>
@@ -40,7 +41,7 @@ import static org.eclipse.persistence.internal.xr.XRDynamicEntity.XR_FIELD_INFO_
  * This API only supports EclipseLink 1.x format deployment XML
  */
 
-@SuppressWarnings({"unchecked"/*, "rawtypes"*/})
+@SuppressWarnings({"rawtypes"})
 public class ProjectHelper {
     /**
      * INTERNAL: Fix the given EclipseLink OR and OX projects so that the
@@ -58,17 +59,17 @@ public class ProjectHelper {
             if (oxProject != null) {
                 xdesc = oxProject.getDescriptorForAlias(desc.getAlias());
             }
-            int idx = 0;
-            XRFieldInfo xrfi = null;
+            XRDynamicPropertiesManager xrDPM = null;
             if (!clz.getName().endsWith(COLLECTION_WRAPPER_SUFFIX)) {
                 try {
-                    Field xrfiField = Helper.getField(clz, XR_FIELD_INFO_STATIC);
-                    xrfi = (XRFieldInfo)xrfiField.get(null);
+                    XRDynamicEntity newInstance = (XRDynamicEntity)clz.newInstance();
+                    xrDPM = newInstance.fetchPropertiesManager();
                 }
                 catch (Exception e) {
                     e.printStackTrace();
                 }
             }
+            Set<String> propertiesNameSet = new HashSet<String>();
             for (Iterator j = desc.getMappings().iterator(); j.hasNext();) {
                 DatabaseMapping dm = (DatabaseMapping)j.next();
                 String attributeName = dm.getAttributeName();
@@ -76,27 +77,27 @@ public class ProjectHelper {
                 if (xdesc != null) {
                     xdm = xdesc.getMappingForAttributeName(attributeName);
                 }
-                dm.setAttributeAccessor(new XRDynamicEntityAccessor(attributeName, idx));
+                dm.setAttributeAccessor(new XRDynamicEntityAccessor(dm));
                 if (xdm != null) {
                     if (dm.isForeignReferenceMapping()) {
                         ForeignReferenceMapping frm = (ForeignReferenceMapping)dm;
                         if (frm.usesIndirection() && frm.getIndirectionPolicy().getClass().
                             isAssignableFrom(BasicIndirectionPolicy.class)) {
-                            xdm.setAttributeAccessor(new XRDynamicEntityVHAccessor(attributeName, idx));
+                            xdm.setAttributeAccessor(new XRDynamicEntityVHAccessor(dm));
                         }
                         else {
                             // no indirection or indirection that is transparent enough (!) to work
-                            xdm.setAttributeAccessor(new XRDynamicEntityAccessor(attributeName, idx));
+                            xdm.setAttributeAccessor(new XRDynamicEntityAccessor(dm));
                         }
                     }
                     else {
-                        xdm.setAttributeAccessor(new XRDynamicEntityAccessor(attributeName, idx));
+                        xdm.setAttributeAccessor(new XRDynamicEntityAccessor(dm));
                     }
                 }
-                if (xrfi != null) {
-                    xrfi.addFieldInfo(attributeName, idx);
-                }
-                idx++;
+                propertiesNameSet.add(attributeName);
+            }
+            if (xrDPM != null) {
+                xrDPM.setPropertyNames(propertiesNameSet);
             }
         }
         // turn-off dynamic class generation
@@ -133,4 +134,4 @@ public class ProjectHelper {
             }
         }
     }
-}
\ No newline at end of file
+}
